import nose.tools as nt
from ckan.common import config
from .test_validation_mocks import WithFrontendValidationMocks, WithAPIValidationMocks
from ckanext.odsh.validation import validate_licenseAttributionByText, validate_extra_groups
import ckan.plugins.toolkit as toolkit


class Test_validate_licenseAttributionByText(WithFrontendValidationMocks):
    def test_it_passes(self):
        validate_licenseAttributionByText(
            self.key_mock,
            self.data_mock_without_groups,
            self.error_mock,
            self.context_mock
        )

    def test_fails_if_licenseAttributionByText_missing(self):
        data_mock = self.data_mock_without_groups.copy()
        data_mock.update({
            ('extras', 4, 'key'): u'',
        })
        with nt.assert_raises(toolkit.Invalid) as err:
            validate_licenseAttributionByText(
                self.key_mock,
                data_mock,
                self.error_mock,
                self.context_mock
            )
        nt.assert_equal(err.exception.error,
                        'licenseAttributionByText: empty not allowed')

    def test_passes_if_licenseAttributionByText_missing_and_flag_set_to_false(self):
        config.update({
            'ckanext.odsh.require_license_attribution': 'False',
        })
        data_mock = self.data_mock_without_groups.copy()
        data_mock.update({
            ('extras', 4, 'key'): u'',
        })
        validate_licenseAttributionByText(
            self.key_mock,
            data_mock,
            self.error_mock,
            self.context_mock
        )

    def test_fails_if_licenseAttributionByText_given_for_license_wo_name(self):
        data_mock = self.data_mock_without_groups.copy()
        data_mock.update({
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
        })
        with nt.assert_raises(toolkit.Invalid) as err:
            validate_licenseAttributionByText(
                self.key_mock,
                data_mock,
                self.error_mock,
                self.context_mock
            )
        nt.assert_equal(
            err.exception.error, 'licenseAttributionByText: text not allowed for this license')


class Test_validate_extra_groupsFrontend(WithFrontendValidationMocks):
    def test_fails_if_required_and_empty_string_in_extras(self):
        data = self.data_mock_without_groups.copy()
        errors = self.error_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(errors.get('groups'), 'at least one group needed')

    def test_passes_if_required_and_groups_in_extras(self):
        data = self.data_mock_with_groups.copy()
        errors = self.error_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(errors.get('groups'), None)

    def test_copies_groups_from_extras_one_level_up(self):
        data = self.data_mock_with_groups.copy()
        errors = self.error_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(
            data.get(('groups', 0, 'id')),
            u'soci'
        )
        nt.assert_equal(
            data.get(('groups', 1, 'id')),
            u'ener'
        )
        nt.assert_equal(
            data.get(('groups', 2, 'id')),
            u'intr'
        )

    def test_adds_empty_group_to_data_if_groups_in_extra_empty(self):
        data = self.data_mock_without_groups.copy()
        errors = self.error_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=False,
            errors=errors
        )
        nt.assert_equal(
            data.get(('groups', 0, 'id')),
            u''
        )


class Test_validate_extra_groupsAPI(WithAPIValidationMocks):
    def test_passes_if_required_and_outside_extras(self):
        data = self.data_mock_with_groups.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(len(errors), 0)

    def test_passes_if_required_and_in_extras(self):
        data = self.data_mock_with_groups_in_extras.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(len(errors), 0)

    def test_fails_if_required_and_not_given(self):
        data = self.data_mock_without_groups.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(len(errors), 1)

    def test_fails_if_required_and_empty(self):
        data = self.data_mock_with_empty_groups.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(len(errors), 1)

    def test_fails_if_required_and_empty_in_extras(self):
        data = self.data_mock_with_empty_groups_in_extras.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=True,
            errors=errors
        )
        nt.assert_equal(len(errors), 1)

    def test_passes_if_not_required_and_outside_extras(self):
        data = self.data_mock_with_groups.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=False,
            errors=errors
        )
        nt.assert_equal(len(errors), 0)

    def test_passes_if_not_required_and_not_given(self):
        data = self.data_mock_without_groups.copy()
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=False,
            errors=errors
        )
        nt.assert_equal(len(errors), 0)

    def test_passes_if_required_and_empty(self):
        data = self.data_mock_with_empty_groups
        errors = self.errors_mock.copy()
        validate_extra_groups(
            data=data,
            requireAtLeastOne=False,
            errors=errors
        )
        nt.assert_equal(len(errors), 0)
